import pytest

from zigzag.api import get_hardware_performance_zigzag

workloads = (
    "zigzag/inputs/workload/resnet18.onnx",
    "zigzag/inputs/workload/resnet18.yaml",
)

# Expected energy and latency for each workload defined above
ens_lats = {
    "zigzag/inputs/workload/alexnet.onnx": (5475639039, 8963572),
    "zigzag/inputs/workload/mobilenetv2.onnx": (952681462, 21868059),
    "zigzag/inputs/workload/resnet18.onnx": (1735241704.2240005, 3998241.0),
    "zigzag/inputs/workload/resnet18.yaml": (2302362179.968, 5736497.0),
}


@pytest.fixture
def mapping():
    return "zigzag/inputs/mapping/tpu_like.yaml"


@pytest.fixture
def accelerator():
    return "zigzag/inputs/hardware/tpu_like.yaml"


@pytest.mark.parametrize("workload", workloads)
def test_api(workload: str, accelerator: str, mapping: str):  # pylint: disable=W0621
    energy, latency, _ = get_hardware_performance_zigzag(workload, accelerator, mapping, exploit_data_locality=True)
    (expected_energy, expected_latency) = ens_lats[workload]
    print(f"'{workload}': ({energy}, {latency}),")
    assert energy == pytest.approx(expected_energy)  # type: ignore
    assert latency == pytest.approx(expected_latency)  # type: ignore
